"""sappgen - Simple App Generator Python

    >>> sappgen [options] <project_name> <app_name>

See https://github.com/aayushuppal/sappgen for more information
"""

# Version of sappgen package
__version__ = "1.0.7"

from .sappgen import process, main



""" Template - 1
project
├── testapp
│   └── util
│   │   ├── __init__.py
│   │   └── log_util.py
│   │── __init__.py
│   └── app.py
├── config.ini
├── main.py
│
├── tests
|   └── test_testapp.py
│
├── Makefile
├── README.md
└── requirements-dev.txt
"""

import os
import logging
from ..util import process_template, cleanup


my_path = os.path.abspath(os.path.dirname(__file__))


class Template1:
    TEMPLATE_PROJ_PATH = f"{my_path[:-10]}/template_files/template1/project"

    def __init__(self, project_name, app_name):
        self.project_name = project_name
        self.app_name = app_name

    def __create_skeleton(self):
        cleanup(self.project_name)

        logging.info(
            f"setting up project structure for {self.project_name}/{self.app_name}"
        )

        os.system(f"mkdir -p {self.project_name}/{self.app_name}/util")
        os.system(f"mkdir -p {self.project_name}/tests")

        os.system(f"touch {self.project_name}/config.ini")
        os.system(f"touch {self.project_name}/main.py")
        os.system(f"touch {self.project_name}/Makefile")
        os.system(f"touch {self.project_name}/README.md")
        os.system(f"touch {self.project_name}/requirements-dev.txt")

        os.system(f"touch {self.project_name}/{self.app_name}/__init__.py")
        os.system(f"touch {self.project_name}/{self.app_name}/app.py")

        os.system(f"touch {self.project_name}/{self.app_name}/util/__init__.py")
        os.system(f"touch {self.project_name}/{self.app_name}/util/log_util.py")

        os.system(f"touch {self.project_name}/tests/test_{self.app_name}.py")

    def __create_files(self):
        logging.info(f"generating files for {self.project_name}/{self.app_name}")

        dest_proj_path = f"{self.project_name}"

        process_template(
            self.project_name,
            self.app_name,
            f"{Template1.TEMPLATE_PROJ_PATH}/config.ini.template",
            f"{dest_proj_path}/config.ini",
        )
        process_template(
            self.project_name,
            self.app_name,
            f"{Template1.TEMPLATE_PROJ_PATH}/main.py.template",
            f"{dest_proj_path}/main.py",
        )
        process_template(
            self.project_name,
            self.app_name,
            f"{Template1.TEMPLATE_PROJ_PATH}/Makefile.template",
            f"{dest_proj_path}/Makefile",
        )
        process_template(
            self.project_name,
            self.app_name,
            f"{Template1.TEMPLATE_PROJ_PATH}/README.md.template",
            f"{dest_proj_path}/README.md",
        )
        process_template(
            self.project_name,
            self.app_name,
            f"{Template1.TEMPLATE_PROJ_PATH}/requirements-dev.txt.template",
            f"{dest_proj_path}/requirements-dev.txt",
        )

        process_template(
            self.project_name,
            self.app_name,
            f"{Template1.TEMPLATE_PROJ_PATH}/testapp/__init__.py.template",
            f"{dest_proj_path}/{self.app_name}/__init__.py",
        )
        process_template(
            self.project_name,
            self.app_name,
            f"{Template1.TEMPLATE_PROJ_PATH}/testapp/app.py.template",
            f"{dest_proj_path}/{self.app_name}/app.py",
        )

        process_template(
            self.project_name,
            self.app_name,
            f"{Template1.TEMPLATE_PROJ_PATH}/testapp/util/__init__.py.template",
            f"{dest_proj_path}/{self.app_name}/util/__init__.py",
        )
        process_template(
            self.project_name,
            self.app_name,
            f"{Template1.TEMPLATE_PROJ_PATH}/testapp/util/log_util.py.template",
            f"{dest_proj_path}/{self.app_name}/util/log_util.py",
        )

        process_template(
            self.project_name,
            self.app_name,
            f"{Template1.TEMPLATE_PROJ_PATH}/tests/test_testapp.py.template",
            f"{dest_proj_path}/tests/test_{self.app_name}.py",
        )

    def process(self):
        self.__create_skeleton()
        self.__create_files()
        logging.info("=== DONE ===")
        os.system(f"tree -a {self.project_name}")



""" Template - 2
project
│
├── testapp
│   ├── routes
│   │   ├── __init__.py
│   │   └── test_routes.py
│   │
│   └── util
│   │   ├── __init__.py
│   │   └── log_util.py
│   │
│   ├── __init__.py
│   └── main.py
│
├── config.ini
├── Makefile
├── README.md
├── requirements.txt
│
└── tests
    └── test_main.py
"""
import os
import logging
from ..util import process_template, cleanup


my_path = os.path.abspath(os.path.dirname(__file__))


class Template2:
    TEMPLATE_PROJ_PATH = f"{my_path[:-10]}/template_files/template2/project"

    def __init__(self, project_name, app_name):
        self.project_name = project_name
        self.app_name = app_name

    def __create_skeleton(self):
        cleanup(self.project_name)

        logging.info(
            f"setting up project structure for {self.project_name}/{self.app_name}"
        )

        os.system(f"mkdir -p {self.project_name}/{self.app_name}/util")
        os.system(f"mkdir -p {self.project_name}/{self.app_name}/routes")
        os.system(f"mkdir -p {self.project_name}/tests")

        os.system(f"touch {self.project_name}/config.ini")
        os.system(f"touch {self.project_name}/Makefile")
        os.system(f"touch {self.project_name}/README.md")
        os.system(f"touch {self.project_name}/requirements.txt")

        os.system(f"touch {self.project_name}/{self.app_name}/__init__.py")
        os.system(f"touch {self.project_name}/{self.app_name}/main.py")

        os.system(f"touch {self.project_name}/{self.app_name}/util/__init__.py")
        os.system(f"touch {self.project_name}/{self.app_name}/util/log_util.py")

        os.system(f"touch {self.project_name}/{self.app_name}/routes/__init__.py")
        os.system(f"touch {self.project_name}/{self.app_name}/routes/test_routes.py")

        os.system(f"touch {self.project_name}/tests/test_main.py")

    def __create_files(self):
        logging.info(f"generating files for {self.project_name}/{self.app_name}")

        dest_proj_path = f"{self.project_name}"

        process_template(
            self.project_name,
            self.app_name,
            f"{Template2.TEMPLATE_PROJ_PATH}/config.ini.template",
            f"{dest_proj_path}/config.ini",
        )
        process_template(
            self.project_name,
            self.app_name,
            f"{Template2.TEMPLATE_PROJ_PATH}/Makefile.template",
            f"{dest_proj_path}/Makefile",
        )
        process_template(
            self.project_name,
            self.app_name,
            f"{Template2.TEMPLATE_PROJ_PATH}/README.md.template",
            f"{dest_proj_path}/README.md",
        )
        process_template(
            self.project_name,
            self.app_name,
            f"{Template2.TEMPLATE_PROJ_PATH}/requirements.txt.template",
            f"{dest_proj_path}/requirements.txt",
        )

        process_template(
            self.project_name,
            self.app_name,
            f"{Template2.TEMPLATE_PROJ_PATH}/testapp/__init__.py.template",
            f"{dest_proj_path}/{self.app_name}/__init__.py",
        )
        process_template(
            self.project_name,
            self.app_name,
            f"{Template2.TEMPLATE_PROJ_PATH}/testapp/main.py.template",
            f"{dest_proj_path}/{self.app_name}/main.py",
        )

        process_template(
            self.project_name,
            self.app_name,
            f"{Template2.TEMPLATE_PROJ_PATH}/testapp/util/__init__.py.template",
            f"{dest_proj_path}/{self.app_name}/util/__init__.py",
        )
        process_template(
            self.project_name,
            self.app_name,
            f"{Template2.TEMPLATE_PROJ_PATH}/testapp/util/log_util.py.template",
            f"{dest_proj_path}/{self.app_name}/util/log_util.py",
        )

        process_template(
            self.project_name,
            self.app_name,
            f"{Template2.TEMPLATE_PROJ_PATH}/testapp/routes/__init__.py.template",
            f"{dest_proj_path}/{self.app_name}/routes/__init__.py",
        )
        process_template(
            self.project_name,
            self.app_name,
            f"{Template2.TEMPLATE_PROJ_PATH}/testapp/routes/test_routes.py.template",
            f"{dest_proj_path}/{self.app_name}/routes/test_routes.py",
        )

        process_template(
            self.project_name,
            self.app_name,
            f"{Template2.TEMPLATE_PROJ_PATH}/tests/test_main.py.template",
            f"{dest_proj_path}/tests/test_main.py",
        )

    def process(self):
        self.__create_skeleton()
        self.__create_files()
        logging.info("=== DONE ===")
        os.system(f"tree -a {self.project_name}")



import os
import logging


def process_template(project_name, app_name, template_file_path, dest_file_path):
    # sed substitute file for PROJECT_NAME & APP_NAME
    # and copy to dest_file_path
    cmd = f"sed \'s/$APP_NAME/{app_name}/g; s/$PROJECT_NAME/{project_name}/g\' {template_file_path} > {dest_file_path}"
    rcode = os.system(cmd)
    if rcode != 0:
        err = f"failed for: {template_file_path} - {dest_file_path}"
        raise Exception(err)


def cleanup(project_name):
    logging.info(f"removing project dir {project_name}")
    os.system(f"rm -rf {project_name}")



"""Simple App Generator - Python

Usage:
------
    $ sappgen [options] <project_name> <app_name>

Create an app -  template1:
    $ sappgen proj app
    OR
    $ sappgen -t1 proj app

Create an app -  template2 - wsgi server app:
    $ sappgen -t2 proj app

Available options are:
    -h, --help         Show this help
    -v, --version      Show package version
    -t1, --template1   Generate application - template 1
    -t2, --template2   Generate wsgi application - template 2

Contact:
--------
- https://aayushuppal.github.io

More information is available at:
- https://pypi.org/project/sappgen
- https://github.com/aayushuppal/sappgen
"""

import logging
import os
import sys
from .cfg import LOG_LEVEL
from .util import set_root_logger_stdout, cleanup
from .templates import Template1, Template2
from . import __version__ as version


set_root_logger_stdout(LOG_LEVEL)


def process(**kwargs):
    logging.info("sappgen:main")
    logging.info(kwargs)

    project_name = "project"
    app_name = "testapp"

    if "project" in kwargs:
        project_name = str(kwargs["project"])
    if "app" in kwargs:
        app_name = str(kwargs["app"])

    tmpl = Template1(project_name, app_name)
    try:
        tmpl.process()
    except:
        logging.exception("setup failed")
        cleanup(project_name)


def processT2(**kwargs):
    logging.info("sappgen:main")
    logging.info(kwargs)

    project_name = "project"
    app_name = "testapp"

    if "project" in kwargs:
        project_name = str(kwargs["project"])
    if "app" in kwargs:
        app_name = str(kwargs["app"])

    tmpl = Template2(project_name, app_name)
    try:
        tmpl.process()
    except:
        logging.exception("setup failed")
        cleanup(project_name)


def main():
    args = [a for a in sys.argv[1:] if not a.startswith("-")]
    opts = [o for o in sys.argv[1:] if o.startswith("-")]

    if "-h" in opts or "--help" in opts:
        print(__doc__)
        print(f"version: {version}")
        return

    if "-v" in opts or "--version" in opts:
        print(f"version: {version}")
        return

    if len(args) != 2:
        print(__doc__)
        print(f"version: {version}")
        return

    project_name = str(args[0])
    app_name = str(args[1])

    if "-t2" in opts or "--template2" in opts:
        processT2(project=project_name, app=app_name)
        return
    elif "-t1" in opts or "--template1" in opts or len(opts) == 0:
        process(project=project_name, app=app_name)
    else:
        print(__doc__)
        print(f"version: {version}")



"""Setup script for sappgen"""

import os.path
from setuptools import setup, find_packages

HERE = os.path.abspath(os.path.dirname(__file__))

with open(os.path.join(HERE, "README.md")) as fid:
    README = fid.read()

setup(
    name="sappgen",
    version="1.0.7",
    description="Simple App Generator - Python",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/aayushuppal/sappgen",
    author="Aayush Uppal",
    author_email="aayuppal@gmail.com",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3"
    ],
    packages=find_packages(exclude=("tests")),
    include_package_data=True,
    install_requires=[
    ],
    entry_points={"console_scripts": ["sappgen=sappgen.sappgen:main"]},
)



import logging
import sys

# ==================================================================================================

LOG_FORMAT = (
    "%(asctime)s.%(msecs)03d %(levelname)s %(filename)s:%(lineno)s - "
    + "%(funcName)s : %(message)s"
)
DATE_FORMAT = "%Y-%m-%d %H:%M:%S"

# ==================================================================================================


def set_root_logger_stdout(log_level):
    logger = logging.getLogger()
    formatter = logging.Formatter(fmt=LOG_FORMAT, datefmt=DATE_FORMAT)

    handler = logging.StreamHandler(stream=sys.stdout)
    handler.setFormatter(formatter)
    logger.addHandler(handler)
    logger.setLevel(log_level)

    return logger
